clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-1-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections


/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 1: Basic OB decomposition</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages (install using command <code>ssc install</code>):
    <code>fre</code>, <code>oaxaca</code>, <code>estout</code>
</p>
***/

webdoc toc 1 1

/***
<p><b>
    In the lecture, a decomposition of the gender wage gap (log wages) was
    shown using data from the GSOEP, where schooling and experience (quadratic
    effect) served as covariates (see slide 18 for data preparation; slides
    28/29 for the decomposition). The data can be found on Ilias
    (<code>gsoep-extract.dta</code>). Using the same data setup, do the
    following.
</b></p>
***/

/***
<h2>Task 1: prepare additional predictors and survey design</h2>
<p><b>
    Extend the <i>X</i> variables of the model by tenure (number of years
    working for the current employer; variable <code>tenure</code>), the
    occupational status as measured by the international socio-economic index
    (<code>isei</code>), and the number of children under the age of 16 in the
    household (<code>children</code>). Also take account of the survey design,
    i.e. primary sampling units (<code>psu</code>), sampling weights
    (<code>weight</code>), and strata (<code>strata</code>).
</b></p>
***/

/***
<p>
    Data preparation (from slides):
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
summarize wage lnwage yeduc expft expft2
drop if missing(sex, lnwage, yeduc, expft)

/***
<p>
    Additional variables: Tenure, ISEI (International Socio-Economic Index of
    Occupational Status), and number of children
</p>
***/

fre tenure, t(5)
fre isei, t(5)
fre children

/***
<p>
    To make sure that the different decompositions below all use the same
    observations, we remove the observations for which any of the additional
    variables is missing.
</p>
***/

drop if missing(tenure, isei, children)

/***
<p>
    Finally, we declare the survey design of the data:
</p>
***/

svyset psu [pw=weight], strata(strata)
svydes

/***
<p>
    There are 5433 respondents and 2035 primary sampling units.
</p>
***/

/***
<h2>Task 2: how additional predictors and survey design affect results</h2>
<p><b>
    Compute the aggregate and detailed Oaxaca-Blinder decomposition. How do
    the results change compared to the specification used in the lecture? (How
    does the survey design change the results? How do the additional variables
    change the results? Make sure to use the same estimation sample when
    comparing results.)
</b></p>
***/

/***
<p>
    We first take a look at how the survey design changes the results.
</p>
<ul><li>
    Original decomposition (slightly different than on slides because some 
    additional observations were excluded):
***/

oaxaca lnwage yeduc (experience: expft expft2), by(sex) weight(1)
estimates store original

/***
</li><li>
    Decomposition taking into account the sampling weights:
***/

oaxaca lnwage yeduc (experience: expft expft2) [pw=weight], by(sex) weight(1)
estimates store weighted

/***
</li><li>
    Decomposition taking into account the full survey design (weights and
    clustering):
***/

oaxaca lnwage yeduc (experience: expft expft2), by(sex) weight(1) svy
estimates store svy

/***
</li><li>
    Comparison:
***/

esttab original weighted svy, mtitles se nostar wide

/***
</li></ul>
<p>
    We see that the sampling weights change the results quite a bit: the
    overall wage gap as well as the explained part decrease and the estimation
    is less precise (larger standard errors). Additionally taking clustering
    and stratification into account does not change the point estimates (there
    would be something wrong if it did), but the standard errors are somewhat
    different (the effect of clustering and stratification on the standard
    errors is only small in this example; this is not generally the case; in
    many situations clustering and stratification can make a big difference).
</p>
***/

/***
<p>
    Now let's include the <b>additional predictors</b> (tenure, isei, and
    children):
</p>
***/

oaxaca lnwage yeduc (experience: expft expft2) tenure isei children, ///
    by(sex) weight(1) svy
estimates store extended

/***
<p>
    Comparison of results:
</p>
***/

esttab svy extended, mtitles se nostar wide

/***
<p>
    Something strange happened. <b>The explained part declined even though we
    included more predictors! How can that be?</b>
</p>
<p>
    In principle, this is perfectly fine because the additional covariates may
    have effects such that the unexplained wage gap increases once we control
    for them (for example, if we include a covariate that has a positive effect
    on wages and for which women have higher values on average than men, such
    that its contribution to the explained part is negative). But in the
    current case this does not seem to make much sense because the added
    variables have positive contributions to the explained part. It is thus a
    bit puzzling why the explained part decreases.
</p>
<p>
    A possible answer is that the extended model is misspecified (i.e., does
    not fit the data very well). Let's try a more flexible model including
    <b>interaction terms between work experience and tenure</b>. Since
    <code>oaxaca</code> does not support factor variable notation
    (<code>oaxaca</code> has been written before factor variable notation was
    introduced to Stata), we have to build the interaction terms manually:
</p>
***/

generate e = expft
generate ee = e*e
generate t = tenure
generate et = e*t
generate eet = e*e*t
generate tt = t*t
generate ett = e*t*t
generate eett = e*e*t*t

/***
<p>
    Check whether we did it right by comparing a regression including the
    manually generated terms with a regression using factor variable notation:
</p>
***/

regress lnwage yeduc e ee t et eet tt ett eett
regress lnwage yeduc c.expft##c.expft##c.tenure##c.tenure, vsquish

/***
<p>
    Seems ok. The results with the manual interaction terms are identical to
    the results using factor notation.
</p>
<p>
    Now we run the decomposition using the flexible specification:
</p>
***/

oaxaca lnwage yeduc e ee t et eet tt ett eett isei children, ///
    by(sex) weight(1) svy nodetail

/***
<p>
    The results are now much closer to the results without <code>tenure</code>
    and <code>isei</code> (the explained part increased from 0.0737 to 0.0883,
    which is close to the original 0.0908). That is, it seems that including
    tenure and work experience in the same models makes things difficult. How
    could we improve the model, while still keeping it simple? Some options:
</p>
<ul><li>
    maybe it would be better not to include tenure at all (but theory speaks
    against that)
</li><li>
    maybe we should include tenure and tenure squared (this does not change
    results much)
</li><li>
    or maybe we should redefine the experience variable to be equal to
    <code>experience - tenure</code> so that there is no overlap between
    experience and tenure (i.e., define experience as the work experience prior
    to starting the current job)
</li></ul>
<p>
The last option seems attractive, so let's try:
</p>
***/

generate exp = expft - tenure
generate exp2 = exp*exp
oaxaca lnwage yeduc (experience: exp exp2) tenure isei children, ///
    by(sex) weight(1) svy

/***
<p>
    Not so bad. The explained part is now .0960. However, looking at the new
    experience variable reveals some serious problems: 
</p>
***/

sum exp, det

/***
<p>
    At least a quarter of all observations have negative values! This happens
    if full-time experience is lower than tenure. There are two reasons for
    that: (a) tenure also captures part-time experience; (b) people report
    length with current employer without taking into account employment
    interruptions due to, for example, parenting. Both reasons are more likely
    among women than among men.
</p>
***/

bysort sex: sum exp, det

/***
<p>
    As can be seen, the negative values in <code>exp</code> are more frequent
    among women than among men. Finding a good model that includes tenure would
    require more careful considerations (e.g., also taking account of part-time
    work experience, etc.). Against this background we will drop tenure from
    our model.
</p>
<p>
    We first reload the data to get observations with missing tenure back
    in.
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
drop if missing(sex, lnwage, yeduc, expft, isei, children)
svyset psu [pw=weight], strata(strata)

/***
<p>
    Our final model is then as follows:
</p>
***/

oaxaca lnwage yeduc (experience: expft expft2) isei children, ///
    by(sex) weight(1) svy

/***
<h2>Task 3: replicate results "by hand"</h2>
<p><b>
    Confirm the results returned by <code>oaxaca</code> (for the extended
    decomposition including survey design) by computing the
    aggregate Oaxaca-Blinder decomposition "by hand" (that is, estimate the
    means of the variables and the regression coefficients and then compute the
    decomposition from these outputs, and not using <code>oaxaca</code>). Also
    compute the contribution of schooling to the "explained" part and the
    "unexplained" part by hand.
</b></p>
***/

/***
<p>
    Computation by hand using matrices (of course, you can also do the
    computations by copying the individual values, like it was done on the
    slides):
</p>
<ul><li>
    store matrices (vectors) containing the coefficients and the means of the
    predictors for males and females (commands such as <code>regress</code> and
    <code>mean</code> store their results in the so-called
    <code>e()</code>-returns; for example, the coefficients/point estimates are
    stored in row vector <code>e(b)</code>)
***/

regress lnwage yeduc expft expft2 isei children [pw=weight] if sex==1
matrix b_m = e(b)
regress lnwage yeduc expft expft2 isei children [pw=weight] if sex==2
matrix b_f = e(b)
mean yeduc expft expft2 isei children [pw=weight] if sex==1 & lnwage<.
matrix X_m = (e(b),1) // appending a "1" for the constant
mean yeduc expft expft2 isei children [pw=weight] if sex==2 & lnwage<.
matrix X_f = (e(b),1) // appending a "1" for the constant

/***
</li><li>
    compute the aggregate decomposition
***/

matrix D = X_m * b_m' - X_f * b_f'
matrix list D
matrix E = (X_m - X_f) * b_m'
matrix list E
matrix U = X_f * (b_m - b_f)'
matrix list U

/***
</li><li>
    compute the contribution of schooling
***/

matrix E_schooling = (X_m[1,"yeduc"] - X_f[1,"yeduc"]) * b_m[1,"yeduc"]
matrix list E_schooling
matrix U_schooling = X_f[1,"yeduc"] * (b_m[1,"yeduc"] - b_f[1,"yeduc"])
matrix list U_schooling

/***
</li></ul>
<p>
    The results are the same as the ones returned by <code>oaxaca</code> above.
</p>
<p>
    Note that the aggregate decomposition can also be computed by generating
    predictions from the models and then taking mean differences. That is, we
    can use so-called out-of-sample predictions to generate the counterfactual
    distribution of interest (the wages that females would get if they were
    paid like men):
</p>
***/

webdoc stlog oom ///
regress lnwage yeduc expft expft2 isei children [pw=weight] if sex==1
predict xb_m if e(sample)
predict xb_fc if sex==2 & lnwage<. // apply male coefficients to female sample
webdoc stlog oom ///
regress lnwage yeduc expft expft2 isei children [pw=weight] if sex==2
predict xb_f if e(sample)
summarize xb_m [aw=weight]  // observed value for men
local xb_m = r(mean)
summarize xb_fc [aw=weight] // counterfactual value for women if paid like men
local xb_fc = r(mean)
summarize xb_f [aw=weight]  // observed value for women
local xb_f = r(mean)
display "difference: "  `xb_m'  - `xb_f'
display "explained: "   `xb_m'  - `xb_fc'
display "unexplained: " `xb_fc' - `xb_f'


