clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-2-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections


/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 2: Postestimation</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages (install using command <code>ssc install</code>):
    <code>oaxaca</code>, <code>estout</code>, <code>coefplot</code>
</p>
***/

webdoc toc 1 1

/***
<p>
    Use the same setup as in Exercise 1 and do the following.
</p>
***/

/***
<h2>Task 1: compute contributions to explained part in percent</h2>
<p><b>
    Compute the percentage contribution of each variable to the explained
    part of the decomposition (including standard errors/confidence intervals)
    for the extended decomposition in Exercise 1.
</b></p>
***/

/***
<p>
    Load data and estimate the decomposition from Exercise 1.
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
drop if missing(sex, lnwage, yeduc, expft, isei, children)
svyset psu [pw=weight], strata(strata)
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1) svy

/***
<p>
    Now compute the percentage contribution using command <code>nlcom</code>;
    this is very similar to the example on the slides.
</p>
***/

nlcom (schooling:  (_b[explained:yeduc]      / _b[overall:explained]) * 100) ///
      (experience: (_b[explained:experience] / _b[overall:explained]) * 100) ///
      (isei:       (_b[explained:isei]       / _b[overall:explained]) * 100) ///
      (children:   (_b[explained:children]   / _b[overall:explained]) * 100)

/***
<h2>Task 2: draw graph of results</h2>
<p><b>
    To illustrate the effect of the survey design, draw a graph that
    displays the above results with and without taking the survey design into
    account.
</b></p>
***/

/***
<p>
    To draw the graph, we use command <code>coefplot</code>, a general-purpose
    command to graph arbitrary estimation results; see
    <a href="http://repec.sowi.unibe.ch/stata/coefplot/index.html">repec.sowi.unibe.ch/stata/coefplot</a>
    for more information. <code>coefplot</code> requires that the results
     are stored as estimation sets; to post result from
    <code>nlcom</code> as an estimation set, apply option <code>post</code>.
</p>
***/

webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1) svy
nlcom (schooling:  (_b[explained:yeduc]      / _b[overall:explained]) * 100) ///
      (experience: (_b[explained:experience] / _b[overall:explained]) * 100) ///
      (isei:       (_b[explained:isei]       / _b[overall:explained]) * 100) ///
      (children:   (_b[explained:children]   / _b[overall:explained]) * 100), post
estimates store svy
webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1)
nlcom (schooling:  (_b[explained:yeduc]      / _b[overall:explained]) * 100) ///
      (experience: (_b[explained:experience] / _b[overall:explained]) * 100) ///
      (isei:       (_b[explained:isei]       / _b[overall:explained]) * 100) ///
      (children:   (_b[explained:children]   / _b[overall:explained]) * 100), post
estimates store nosvy
coefplot svy nosvy, recast(bar) barwidth(0.3) citop ciopts(recast(rcap)) xline(0) xlab(#10)
webdoc graph

/***
<h2>Task 3: create Word table of results</h2>
<p><b>
    Create a Word table (<code>rtf</code> file type in <code>esttab</code>)
    of the underlying decomposition results (with and without survey design)
    that could be included in the appendix of your paper.
</b></p>
***/

/***
<p>
    Command <code>esttab</code> can be used to compile a table from multiple estimation
    results and export the table to different formats; see 
    <a href="http://repec.sowi.unibe.ch/stata/estout/index.html">repec.sowi.unibe.ch/stata/estout</a>
    for more information.
</p>
***/

webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1) svy
estimates store svy
webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1)
estimates store nosvy
esttab svy nosvy using Exercise-2-table.rtf, replace ///
    nostar b(3) se wide nonumber drop(group_*) ///
    mtitle("With survey design" "Without survey design") ///
    eqlabels(, prefix("{\i ") suffix("}"))

/***
<p>
    Opening the table in Word looks about as follows.
</p>
***/

webdoc put <img alt="Exercise-2-table" src="data:image/png;base64,
webdoc append `"Exercise-2-table.base64"'
webdoc put "/>

/***
<p>
    In Stata 17, the <code>collect</code> command has been introduced
    as an official tool to compile and export tables from arbitrary results.
    The command looks very promising, but has a rather steep learning curve.
    Here is an example producing a table similar to the one above.
</p>
***/

collect clear
webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1) svy
collect get _r_b _r_se, tag(model["With survey design"])
webdoc stlog oom ///
oaxaca lnwage yeduc (experience: expft expft2) isei children, by(sex) weight(1)
collect get _r_b _r_se, tag(model["Without survey design"])
collect style column, dups(center)
collect style cell, nformat(%6.3f)
collect style cell result[_r_se], sformat("(%s)")
collect style header result, level(hide)
collect style header colname, level(value)
collect style cell border_block, border(right, pattern(nil))
collect layout (coleq#colname) (model#result)
collect export Exercise-2-table-2.docx, replace

/***
<p>
    The table looks as follows in Word.
</p>
***/

webdoc put <img alt="Exercise-2-table-2" src="data:image/png;base64,
webdoc append `"Exercise-2-table-2.base64"'
webdoc put "/>

/***
<p>
    Alternatively, you could use command <code>etable</code>, which is a wrapper
    for <code>collect</code>.
</p>
***/
