clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-3-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 3: Index and transformation problem</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages (install using command <code>ssc install</code>):
    <code>fre</code>, <code>oaxaca</code>, <code>estout</code>
</p>
***/

webdoc toc 2 1

/***
<h2>Data preparation</h2>
***/

// get data as in Exercise 1
use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
drop if missing(sex, lnwage, yeduc, expft, isei, children)
svyset psu [pw=weight], strata(strata)

/***
<h2>Part I: The index problem</h2>
***/

/***
<h3>Task 1: choice of reference coefficients</h3>
<p><b>
    Using the extended decomposition from Exercise 1 (i.e. schooling,
    full-time experience, ISEI, number of children), evaluate how the results
    change depending on how you handle the index problem. Compute the following
    variants:
    <ul>
    <li>male coefficients</li>
    <li>female coefficients</li>
    <li>pooled model</li>
    <li>threefold decomposition</li>
    </ul>
</b></p>
***/

/***
<p>
    Using male coefficients as "nondiscriminatory" coefficients: set option
    <code>weight()</code> to 1; this gives the coefficients from the first
    group (males in this case) a weight of one and the coefficients from the
    second group (females) a weight of zero.
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy weight(1)
estimates store male

/***
<p>
    Using female coefficients as "nondiscriminatory" coefficients: set option
    <code>weight()</code> to 0.
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy weight(0)
estimates store female

/***
<p>
    Pooled model: apply option <code>pooled</code> instead of using the 
    <code>weight()</code> option (the pooled model will automatically include
    a group dummy; if you want to use a pooled model without group dummy, you
    can apply option <code>omega</code> instead of <code>pooled</code>).
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy pooled
estimates store pooled

/***
<p>
    Threefold decomposition (view of women): omit <code>weight()</code> and
    <code>pooled</code> (<code>threefold</code> is the default).
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy
estimates store tf_female

/***
<p>
    Threefold decomposition (view of men): add <code>threefold(reverse)</code>.
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy threefold(reverse)
estimates store tf_male

/***
<h3>Task 2: overview table</h3>
<p><b>
    Generate an overview table and try to make sense of the results. What is
    the correct interpretation of the various results? How can the differences
    be explained?
</b></p>
***/

esttab male female pooled tf_female tf_male, se  nonumber mtitles ///
    equations(Overall=1, Explained=2, Unexplained=3) ///
    rename(endowments explained coefficients unexplained)

/***
<p>
    Explanation for use of options <code>equations()</code> and
    <code>rename()</code>: A complication when compiling an overview table is
    that the different parts in the output are labeled differently across the
    decompositions. In the two-fold decomposition label "explained" is used for
    &Delta;<sub>X</sub> and "unexplained" for &Delta;<sub>S</sub>. In the
    three-fold decomposition the corresponding labels are "endowments" and
    "coefficients". By default <code>esttab</code> places differently named
    elements into different rows, which results in a messy table in the current
    case. To tidy up the table, option <code>equations()</code> specifies how
    equations be merged and option <code>rename()</code> renames some of the
    coefficients.
</p><p>
    Interpretation: The choice of the reference coefficients changes results
    somewhat. When using the male coefficients, the explained part
    of the gender wage gap is larger than when using the female
    coefficients (using male coefficients, 0.104/0.206 = 50% of the wage gap is
    explained; using female coefficients, only 0.0856/0.206 = 42% is explained).
    The difference is mostly due to the steeper earnings profile of men across
    work experience. Because of that, the gender difference in work experience
    explains more of the overall wage gap if male coefficients are used as
    reference coefficients. This can also nicely be seen in the "interaction" equation of
    the three-fold decomposition that quantifies the differences in the
    contributions to the explained part depending on whether the male or the
    female coefficients are used as reference (only for experience this
    difference is substantial). Using the pooled model leads
    to a compromise between the two extremes.
</p>
<p>
    Fundamentally, the difference between using the male coefficients and the
    female coefficients is a change in perspective in the sense that different
    counterfactual exercises are performed. When using the male coefficients,
    we essentially ask how much men would lose if their work experience was
    reduced to that of women. When using the female coefficients, we ask how
    much women would gain if their work experience was increased to that
    of men. (While assuming that everything else stays the same.)
</p>
***/


/***
<h3>Task 3: average treatment effect</h3>
<p><b>
    Optional: Compute a decomposition that is defined in a way such that the
    unexplained component can be interpreted as an "average treatment effect"
    (see slides for details).
</b></p>
***/

/***
<p>
    Giving the male coefficients a weight equal to the proportion of females in
    the sample, and giving the female coefficients a weight equal to the
    proportion of males, leads to an unexplained part that is equal in size to
    the average treatment effect obtained by a regression-adjustment estimator:
</p>
***/

svy: proportion sex
local p_female = _b[2.sex]
oaxaca lnwage yeduc (exp: expft expft2) isei children, by(sex) svy weight(`p_female') nodetail

/***
<p>
    Confirm using <code>teffects ra</code> (the command does not support survey
    estimation, but we can still take account of sampling weights and clustering;
    the only element of the survey design we then ignore is stratification; this
    seems acceptable; we are only interested here in the point estimate anyhow,
    which is not affected by clustering and stratification):
</p>
***/

teffects ra (lnwage yeduc expft expft2 isei children) (sex) [pw=weight], vce(cluster psu)


/***
<h2>Part II: The transformation problem</h2>
***/

/***
<h3>Task 1: decomposition with categorical predictor</h3>
<p><b>
    1. Replace ISEI in the extended model of Exercise 1 by the (categorical)
    EGP variable (<code>egp</code>). Before you do that, inspect the variable
    <code>egp</code> carefully and drop categories with a very low number of
    observations. Only report the aggregate contribution of EGP. Illustrate how
    the results change if you switch the base level.
</b></p>
***/

/***
<p>
    Make dummies (we do not use <code>tabulate, generate()</code> here because we
    want to name the dummies using the corresponding EGP values;
    <code>tabulate, generate()</code> would use consecutive numbers):
</p>
***/

fre egp
drop if egp==5 // only two observations; self-employed are not part of gsoep-extract.dta
quietly levelsof egp
foreach l in `r(levels)' {
    quietly generate byte egp_`l' = egp==`l' if egp<.
}
summarize egp_*

/***
<p>
    Results using class I (upper service class) as reference category:
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) children (EGP: egp_2-egp_8), by(sex) weight(1) svy

/***
<p>
    Results using class V+VI (skilled manual workers) as reference category:
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) children (EGP: egp_1-egp_4 egp_7 egp_8), ///
    by(sex) weight(1) svy

/***
<p>
    The contribution of EGP to the explained part does not change. However, the
    contribution to the unexplained part changes quite dramatically depending
    on the choice of the reference category.
</p>
***/


/***
<h3>Task 2: normalized decomposition</h3>
<p><b>
    Normalize the effects of EGP to make its contribution independent of the
    choice of the base level (unweighted normalization using
    <code>oaxaca</code>).
</b></p>
***/

oaxaca lnwage yeduc (exp: expft expft2) children (EGP: normalize(egp_*)), ///
    by(sex) weight(1) svy

/***
<h3>Task 3: how collapsing categories changes results</h3>
<p><b>
    Now simplify the EGP variable by combining classes VIIa and VIIb (codes 7
    and 8) into one bigger class. How do the decomposition results change?
</b></p>
***/

generate byte EGP = egp
replace EGP = 7 if EGP==8
fre EGP
quietly levelsof EGP
foreach l in `r(levels)' {
    quietly generate byte EGP_`l' = EGP==`l' if EGP<.
}
summarize EGP_*
oaxaca lnwage yeduc (exp: expft expft2) children (EGP: normalize(EGP_*)), ///
    by(sex) weight(1) svy

/***
<p>
    The contribution to the explained part did not change very much. However,
    for the unexplained part, the contribution of EGP is different (-0.046
    vs. -0.027). This is a general problem of the (unweighted) normalization:
    for the contribution to the unexplained part, results can change quite a
    bit depending on minor changes in the categories.
</p>
***/


/***
<h3>Task 4: weighted normalization</h3>
<p><b>
    Optional: Compute the contribution of EGP and the simplified EGP to the
    unexplained part using a weighted normalization. You need to do this
    manually (hint: you can use command <code>contrast</code> to obtain
    normalized coefficients after running a regression). Compare the results to
    the results from the unweighted normalization.
</b></p>
***/

/***
<p>
    This is currently not implemented in <code>oaxaca</code> and has to be done
    manually. The approach is to first use <code>contrast</code> to compute the
    transformed coefficients and then use matrix multiplication to obtain the
    contribution to the unexplained part. The <code>gw.</code> operator is what
    we need to compute deviation contrasts from the weighted mean.
</p>
***/

webdoc stlog oom ///
svy, subpop(if sex==1): ///
    regress lnwage yeduc expft expft2 children i.egp, nofvlab
contrast gw.egp, nofvlab nowald
matrix b_m = r(b)
webdoc stlog oom ///
svy, subpop(if sex==2): ///
    regress lnwage yeduc expft expft2 children i.egp, nofvlab
contrast gw.egp, nofvlab nowald
matrix b_f = r(b)
svy, subpop(if sex==2): proportion egp if e(sample)
matrix X_f = e(b)
matrix U   = X_f * (b_m - b_f)'
matrix list U

/***
<p>
    We now pack this into a small program so we can re-use it and so that it also
    supports the unweighted normalization (operator <code>g.</code> instead of
    <code>gw.</code>).
</p>
***/

capture program drop egpdecomp
program egpdecomp, rclass
    args op egp
    quietly {
        svy, subpop(if sex==1): ///
            regress lnwage yeduc expft expft2 children i.`egp', nofvlab
        contrast `op'.`egp', nofvlab nowald
        matrix b_m = r(b)
        svy, subpop(if sex==2): ///
            regress lnwage yeduc expft expft2 children i.`egp', nofvlab
        contrast `op'.`egp', nofvlab nowald
        matrix b_f = r(b)
        svy, subpop(if sex==2): proportion `egp' if e(sample)
        matrix X_f = e(b)
        matrix U   = X_f * (b_m - b_f)'
    }
    return scalar U = U[1,1]
    display as txt "Contribution to unexplained part = " as res return(U)
end

/***
<p>
    Using this program, results for different situations can be computed without
    much effort:
</p>
***/

egpdecomp g egp
egpdecomp g EGP
egpdecomp gw egp
egpdecomp gw EGP

/***
<p>
    The first two results are the same as above using the unweighted
    normalization. The latter two are the results using the weighted
    normalization. As can be seen, the change in the categorization
    (<code>egp</code> vs. <code>EGP</code>) only has a minor effect on the
    results using weighted normalization.
</p>
<p>
    Looking at the results we also realize that in case of the weighted
    normalization the aggregate contribution of a categorical predictor to the
    unexplained part is exactly -1 times the contribution to the explained part!
    This is a formal property of the weighted normalization (at least in a
    linear decomposition)!
</p>
<p>
    This means that we do not really need to compute the weighted normalization;
    we can just read it off the standard output from <code>oaxaca</code> (i.e.
    -1 * the contribution to the explained part). However, it also highlights
    once more that the detailed decomposition of the unexplained part is
    problematic (what is the point of computing the contribution to the
    unexplained part if we know that mechanically it will just be -1 times the
    contribution to the explained part?).
</p>
<p>
    Note that Kim (2013) suggests a slightly different weighted normalization.
    Using <code>contrast</code> with the <code>gw.</code> operator is
    equivalent to normalizing the coefficients of each model using the
    distribution of categories in the (sub)sample that has been used to
    estimate the model. What Kim (2013) suggests is to use the distribution of
    categories <i>in the overall sample across both groups</i> to normalize the
    coefficients of each model. This leads to slightly different results such
    that the relation above (i.e., that the contribution to the unexplained
    part is equal to -1 times the contribution to the explained part) only
    holds approximately.
</p>
***/

/***
<h3>Task 5: industry decomposition</h3>
<p><b>
    Optional: Compute the "industry decomposition" described on the
    slides by economic sector (variable <code>industry</code>).
</b></p>
***/

/***
<p>
    Define dummy variables for the three sectors:
</p>
***/

fre industry
generate byte primary   = industry==1 if industry<.
generate byte secondary = industry==2 if industry<.
generate byte tertiary  = industry==3 if industry<.

/***
<p>
    Run the decomposition using the secondary sector as base level:
</p>
***/

oaxaca lnwage yeduc (exp: expft expft2) children primary tertiary, ///
    by(sex) weight(1) svy

/***
<p>
    Compute the industry decomposition by Horrace and Oaxaca (2001):
</p>
***/

matrix coefs = e(b0)
matrix I = J(3,1,.)
matrix rownames I = Primary Secondary Tertiary
matrix I[2,1] = _b[overall:unexplained] - _b[unexplained:primary] - _b[unexplained:tertiary]
matrix I[1,1] = I[2,1] + (coefs[1,"b1:primary"]  - coefs[1,"b2:primary"])
matrix I[3,1] = I[2,1] + (coefs[1,"b1:tertiary"] - coefs[1,"b2:tertiary"])
matrix list I

/***
<p>
    Compute the contributions of the sectors to unexplained wage gap according to
    Fortin et al. (2011):
</p>
***/

svy, subpop(if sex==2): mean primary secondary tertiary if lnwage<.
matrix p = e(b)
local S = p[1,1]*I[1,1] + p[1,2]*I[2,1] + p[1,3]*I[3,1]
display `S'    // equal to total unexplained
matrix I[1,1] = p[1,1]*I[1,1] / `S'
matrix I[2,1] = p[1,2]*I[2,1] / `S'
matrix I[3,1] = p[1,3]*I[3,1] / `S'
matrix list I

/***
<p>
    We can also use <code>nlcom</code> to compute these decompositions. The
    advantage is that in this way we will also get standard errors and
    confidence intervals. The procedure goes as follows. Apart from the
    decomposition results, <code>oaxaca</code> also returns the underlying
    regression coefficients and means as well as their joint variance matrix in
    <code>e(b0)</code> and <code>e(V0)</code>.
</p>
***/

webdoc stlog oom ///
oaxaca lnwage yeduc (exp: expft expft2) children primary tertiary, ///
    by(sex) weight(1) svy
matrix list e(b0)

/***
<p>
    To be able to apply <code>nlcom</code> to these results, we first need to
    post the results as a new estimation set using <code>ereturn post</code>:
</p>
***/

matrix b = e(b0)
matrix V = e(V0)
ereturn post b V

/***
<p>
    We can now piece together the expressions to be submitted to
    <code>nlcom</code>. To reduce writing, we use a loop over the variables to
    collect the elements that are part of each expression:
</p>
***/

local ref (_b[b1:_cons]-_b[b2:_cons])
foreach v in yeduc expft expft2 children {
    local ref `ref' + (_b[b1:`v']-_b[b2:`v'])*_b[x2:`v']
}
nlcom (Primary:   `ref' + (_b[b1:primary]-_b[b2:primary]))   ///
      (Secondary: `ref')                                     ///
      (Tertiary:  `ref' + (_b[b1:tertiary]-_b[b2:tertiary])) ///
      , noheader

/***
<p>
    Likewise, the rescaled variant as suggested by Fortin et al. (2011) can be
    computed as follows:
</p>
***/

local ref (_b[b1:_cons]-_b[b2:_cons])
foreach v in yeduc expft expft2 children {
    local ref `ref' + (_b[b1:`v']-_b[b2:`v'])*_b[x2:`v']
}
local p1        (_b[x2:primary])
local p2        (1 - _b[x2:primary] - _b[x2:tertiary])
local p3        (_b[x2:tertiary])
local primary   (`ref' + (_b[b1:primary]-_b[b2:primary])) * (`p1')
local secondary (`ref') * (`p2')
local tertiary  (`ref' + (_b[b1:tertiary]-_b[b2:tertiary])) * (`p3')
local sum       (`primary' + `secondary' + `tertiary')
nlcom (Primary:     `primary'/`sum') ///
      (Secondary: `secondary'/`sum') ///
      (Tertiary:   `tertiary'/`sum') ///
      , noheader

