clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-4-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 4: Nonlinear decomposition</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages: <code>fre</code>, <code>estout</code>,
    <code>oaxaca</code>, <code>nldecompose</code>, <code>fairlie</code>
</p>
***/

/***
<p><b>
    Extend the example analysis from the slides (<code>4-nonlinear.pdf</code>)
    by X variables “locus of control” (<code>LoC</code>) and “willingness to
    take risk” (<code>risk</code>). Compute the aggregate and detailed
    decomposition using the Fairlie, Yun and LPM decomposition for non-linear
    models and interpret the results.
</b></p>
***/

/***
<p>
    Set the seed of the random number generator for sake of reproducibility:
</p>
***/

set seed 5432334

/***
<p>
    Data preparation as on slides:
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate byte male = sex==1
generate byte female = 1 - male
summarize supvis yeduc expft exppt male


/***
<p>
    Additional predictors:
</p>
***/

// locus of control
fre LoC, t(5)
// willingness to take risks
fre risk
// summarize
summarize LoC risk

/***
<p>
    Drop observations with missing values and set survey design:
</p>
***/

drop if missing(supvis, yeduc, expft, exppt, LoC, risk)
svyset psu [pw=weight], strata(strata)

/***
<p>
    Run a logistic regression to check whether the added variables are relevant
    for the distinction between supervising and not supervising:
</p>
***/

svy: logit supvis yeduc expft exppt LoC risk

/***
<p>
    Both variables seem to be relevant: As may have been expected, people
    working as supervisors/in leadership positions seem to be more willing to
    take risks. Moreover, their locus of control is less likely to be
    externally oriented when compared to people not working as supervisors.
</p>
<p>
    Now have a look at gender differences:
</p>
***/

svy: mean LoC risk, over(sex)

/***
<p>
    Women seem to be slightly more externally oriented and are considerably
    more risk averse than men. We thus might expect that these variables will
    "explain" at least some of the gender gap in supervision.
</p>
<p>
    We can also look at gender differences in coefficients:
</p>
***/

svy, subpop(if sex==1): ///
    logit supvis yeduc expft exppt LoC risk, nolog
svy, subpop(if sex==2): ///
    logit supvis yeduc expft exppt LoC risk, nolog

/***
<p>
    Locus of control seems to have an effect only among men but not among
    women. The effect of risk taking is very similar for men and women.
</p>
<p>
    We now turn to the decompositions. First we run the aggregate decomposition
    with and without the added variables, using the <code>nldecompose</code>
    command:
</p>
***/

// reduced model
nldecompose, by(male): svy: logit supvis yeduc expft exppt 
//  full model
nldecompose, by(male): svy: logit supvis yeduc expft exppt LoC risk

/***
<p>
    The overall gender gap is about 15 percentage points (i.e. for males, the
    proportion working as supervisors/in leadership positions is about 15
    percentage points higher than for females). Gender differences in schooling
    as well as full-time and part-time experience partly explain this
    difference (34% or 15% of the gap, depending on whether male or the female
    coefficients are used as reference; interestingly, the explained part is
    larger if we used the male coefficients as reference; the effects of
    the predictors thus seem to be stronger in the male sample).
</p>
<p>
    Adding "locus of control" and "risk taking" increases the explained part
    to 21–42% of the overall gap. It is still the case that the explained part is
    larger if we used the male coefficients as reference.
</p>
<p>
    To obtain a detailed decomposition we now run Farlie decompositions (with
    random ordering), Yun decompositions, as well as decompositions based on
    the linear probability model (LPM):
</p>
***/

// Fairlie
// - male coefficients as reference
fairlie supvis yeduc expft exppt LoC risk [pw=weight], by(female) noest ///
    ro reps(1000) nodots
est sto fairlie_m
// - female coefficients as reference
fairlie supvis yeduc expft exppt LoC risk [pw=weight], by(female) noest ///
    ro reps(1000) nodots reference(1)
est sto fairlie_f
// Yun
// - male coefficients as reference
oaxaca supvis yeduc expft exppt LoC risk, by(female) weight(1) logit svy
est sto yun_m
// - female coefficients as reference
oaxaca supvis yeduc expft exppt LoC risk, by(female) weight(0) logit svy
est sto yun_f
// LPM
// - male coefficients as reference
oaxaca supvis yeduc expft exppt LoC risk, by(female) weight(1) svy
est sto LPM_m
// - female coefficients as reference
oaxaca supvis yeduc expft exppt LoC risk, by(female) weight(0) svy
est sto LPM_f

/***
<p>
    Overview of the results:
</p>
***/

esttab fairlie_m yun_m LPM_m fairlie_f yun_f LPM_f, ///
    compress varw(12) equations(Explained=1:2:2:1:2:2) mtitle nonumber ///
    keep(Explained: overall:difference overall:explained overall:unexplained) ///
    mgroup("Male coefficients as reference" "Female coefficients as reference", ///
        pattern(1 0 0 1 0 0) span)

/***
<p>
    The results from the different methods are very similar. When using the
    male coefficients as reference, locus of control and risk taking explain a
    larger part of the gap than when using the female coefficients as
    reference. Given the total gap of about 15 percentage points, however,
    these contributions still seem rather moderate.
</p>
***/



