clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-5-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 5: Difference-in-differences decomposition</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages: <code>estout</code>, <code>oaxaca</code>,
    <code>smithwelch</code>, <code>xtoaxaca</code>
</p>
***/

webdoc toc 1 1

/***
<p>
    Set the seed of the random number generator for sake of reproducibility:
</p>
***/

set seed 432987

/***
<h2>Task 1: Smith-Welch decomposition</h2>
<p><b>
    Repeat the Smith-Welch example analysis from the slides
    (<code>5-did.pdf</code>) and evaluate how changing the reference and
    “benchmark” estimates (as they are called in the help file) affects the
    results. Try to provide clear interpretations of the different elements of
    the output and explain how the interpretations change depending on the
    choice of the reference and benchmark estimates.
</b></p>
***/

/***
<p>
    Data preparation as on slides:
</p>
***/

use gsoep-extract, clear
keep if inlist(wave,1995,2015)
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
generate byte t = wave==2015   // 0 = 1995, 1 = 2015
generate byte female = sex==2  // 0 = male, 1 = female
summarize lnwage yeduc expft expft2 t female
drop if missing(lnwage,yeduc,expft) // remove unused observation
svyset psu [pw=weight], strata(strata)

/***
<p>
    The <code>benchmark()</code> option selects the sample from which the
    reference coefficients should be taken, i.e. either from the 1995 sample or
    from the 2015 sample. In the example on the slides,
    <code>benchmark()</code> was set such that the reference coefficients are
    taken from the 1995 sample. Furthermore, the <code>reference()</code>
    option sets whether the reference coefficients are taken from males or from
    females. In the example on the slides, <code>reference()</code> was set
    such that male coefficients are used.
</p>
<p>
    Results will depend on these settings, just like the results of the
    standard OB decomposition depend on the choice of the reference
    coefficients. We will now compute results for four different combinations
    (t=1995 vs. t=2015 and male vs. female).
</p>
<p>
    First we need to estimate the four regression models:
</p>
***/

// t=1995
svy: regress lnwage yeduc expft expft2 if female==0 & t==0
estimates store male_t0
svy: regress lnwage yeduc expft expft2 if female==1 & t==0
estimates store female_t0
// t=2015
svy: regress lnwage yeduc expft expft2 if female==0 & t==1
estimates store male_t1
svy: regress lnwage yeduc expft expft2 if female==1 & t==1
estimates store female_t1

/***
<p>
    We can no run the four variants of the decomposition. For easy comparison, 
    we also store the results in a matrix.
</p>
***/

capture matrix drop results
// benchmark: t=1995; reference: male
smithwelch male_t0 female_t0 male_t1 female_t1, benchmark(1) reference(1)
matrix results = r(DD)'
// benchmark: t=1995; reference: female
smithwelch male_t0 female_t0 male_t1 female_t1, benchmark(1) reference(2)
matrix results = results, r(DD)'
// benchmark: t=2015; reference: male
smithwelch male_t0 female_t0 male_t1 female_t1, benchmark(2) reference(1)
matrix results = results, r(DD)'
// benchmark: t=2015; reference: female
smithwelch male_t0 female_t0 male_t1 female_t1, benchmark(2) reference(2)
matrix results = results, r(DD)'

/***
<p>
    First, note that the gender-wage gap is somewhat smaller in 2015 than in
    1995 (0.205 vs. 0.221; difference of 0.015). Furthermore, have a look at the
    table labeled "Decompositions of individual differentials". The choice of
    the benchmark sample has no effect on these results because they are
    separate Oaxaca-Blinder decompositions by time point (1995 vs. 2015). The
    results, however, depend on whether the male coefficients or the
    female coefficients are used as reference coefficients. At both time
    points, the explained part is larger with the male coefficients than with
    the female coefficients (0.076 vs. 0.059 in 1995; 0.090 vs. 0.074 in 2015).
    This is mostly due to the fact that men have steeper wage profiles across
    work experience at both time points.
</p>
<p>
    Now have a look at the other results. The overview is as follows (the
    matrix also contains results labeled as "EC" and "dEC", which are all zero;
    these components are interaction terms that are only relevant in three-fold
    decompositions, i.e. when the <code>reference()</code> and/or
    <code>benchmark()</code> options are omitted):
</p>
***/

matrix colnames results = "1995:male" "1995:female" "2015:male" "2015:female"
matlist results

/***
<p>
    The rows "dE:D" and "dC:D" show the time differences in the explained part
    and in the unexplained part. A positive value means that the corresponding
    component is larger in 2015 than in 1995. We see that the "dE:D" component
    is positive, that is, the explained wage gap is larger in 2015 than in
    1995. At the same time, the unexplained wage gap has become smaller in 2015
    when compared to 1995 (negative "dC:D" component). Moreover, in terms of
    magnitude, the change in the unexplained part is more pronounced than the
    change in the explained part.
</p>
<p>
    The exact breakdown depends on whether the male coefficients or the female
    coefficients are used as the reference coefficients, but the differences
    are very small (0.0146 vs. 0.0149 for the explained part; -0,0300 vs.
    -0.0303 for the unexplained part; the choice of the benchmark sample is
    irrelevant for these results). That is, although the choice of the
    reference coefficients is relevant for the division between the explained
    part and the unexplained part of the wage gap (see above), the time
    differences in these components remain stable. This is because the change
    from male to female reference coefficients has a similar effect in both
    time points.
</p>
<p>
    <strong>The main terms of interest of the Smith-Welch decomposition are the
    terms "dE:E" and "dC:C".</strong> Term "dE:E" indicates how much of the
    difference in the explained part between 2015 and 1995 is because men and
    women are, in fact, more similar or dissimilar to each other in terms of
    their wage predictors in 1995 than in 2015. We see that the value is
    negative, meaning that, in total, men and women became more similar over
    time. (But: As we know, women became "more similar" to men by surpassing
    them with respect to schooling, while at the same time losing ground with
    respect to full-time experience.) Term "dE:E" is a bit more pronounced if
    we use female coefficients as the reference coefficients. Since this
    component has a negative sign, we conclude that changes in the
    gender gap in wage predictors per se has led to a narrowing gender wage
    gap.
</p>
<p>
    The other component of the difference in the explained part (term "dE:C")
    is positive because a given male–female gap in predictors is more relevant
    for the gender wage gap in 2015 than in 1995 (i.e., the gap in predictors
    in 2015 or 1995 is multiplied by larger coefficients). This is due to the
    fact that for both men and women returns to education increased and wage
    profiles across work experience got steeper between 1995 and 2015. Term
    "dE:C" is a bit less pronounced if we use male coefficients as the
    reference coefficients. Likewise, the terms is a bit less pronounced when
    using 1995 as "benchmark". Term "dE:C" is larger than "dE:E", leading to an
    overall increase in the explained part of the decomposition over time. That
    is, even though the gender gap in predictors decreased (negative "dE:E"),
    the explained part of the decomposition increased because the gap in
    predictors became more relevant due to increasing effects of the predictors-
</p>
<p>
    Term "dC:C" indicates how much of the difference in the unexplained part
    between 2015 and 1995 is because the male–female difference in coefficients
    is more/less pronounced in 2015 than in 1995. As the term is negative, we
    conclude, that between 1995 and 2015 men and women became more similar with
    respect to the gender-specific wage structure (i.e. the regression
    coefficients). We see that most of the decline in the unexplained part over
    time is due to this.
</p>
<p>
    The other component of the difference in the unexplained part (term "dC:E")
    is also (mostly) negative. This implies that changes in male or female wage
    predictors would have narrowed the gender wage gap if male–female
    difference in coefficients had stayed at their 1995/2015-level. Choosing
    male coefficients as the reference as well as choosing 1995 as "benchmark"
    results in somewhat stronger effects for this component.
</p>
***/

/***
<h2>Task 2: bootstrap standard errors</h2>
<p><b>
    Compute bootstrap standard errors for the different decompositions.
</b></p>
***/

/***
<p>
    The whole process – the estimation of the regression models and the
    subsequent decomposition – needs to be bootstrapped. That is, we need to
    write a little program that estimates all models, then applies
    <code>smithwelch</code>, and then posts the results in a way such that
    <code>bootstrap</code> can easily collect them. The program could look
    about as follows.
</p>
***/

webdoc stlog, cmdlog
capt prog drop mysmithwelch
program mysmithwelch, eclass
    regress lnwage yeduc expft expft2 [pw=weight] if female==0 & t==0
    estimates store male_t0
    regress lnwage yeduc expft expft2 [pw=weight] if female==1 & t==0
    estimates store female_t0
    regress lnwage yeduc expft expft2 [pw=weight] if female==0 & t==1
    estimates store male_t1
    regress lnwage yeduc expft expft2 [pw=weight] if female==1 & t==1
    estimates store female_t1
    smithwelch male_t0 female_t0 male_t1 female_t1 `0'
    matrix b = r(DD)
    ereturn post b
end
webdoc stlog close

/***
<p>
    Remarks:
</p>
<ul>
    <li>
        <code>smithwelch</code> stores its results in <code>r()</code>; see the
        help file for details. For <code>bootstrap</code>, however, it is
        easiest if the results to be bootstrapped are returned in vector
        <code>e(b)</code>. Posting results in <code>e()</code> can be
        accomplished using command <code>ereturn post</code>; a prerequisite
        for using <code>ereturn post</code> is that the program is declared as
        an "eclass" program.
    </li>
    <li>
        Only the results displayed in the table "Decomposition of difference in
        differentials" (which are stored by <code>smithwelch</code> in matrix
        <code>r(DD)</code>) are included in the bootstrap, as these results are
        the main results of interest in the Smith-Welch decomposition. Of
        course, other results could be included by modifying the program.
    </li>
    <li>
        Any options specified with <code>mysmithwelch</code> will be passed
        through to <code>smithwelch</code>; this allows us to specify the
        <code>benchmark()</code> and <code>reference()</code> options.
    </li>
</ul>
<p>
    We can now use the program to obtain bootstrap estimates for the different 
    variants of the decomposition:
</p>
***/

bootstrap, cluster(psu) strata(strata) reps(100) nowarn: mysmithwelch, benchmark(1) reference(1)
estimates store b1995m
bootstrap, cluster(psu) strata(strata) reps(100) nowarn: mysmithwelch, benchmark(1) reference(2)
estimates store b1995f
bootstrap, cluster(psu) strata(strata) reps(100) nowarn: mysmithwelch, benchmark(2) reference(1)
estimates store b2015m
bootstrap, cluster(psu) strata(strata) reps(100) nowarn: mysmithwelch, benchmark(2) reference(2)
estimates store b2015f

/***
<p>
    <strong>Important remark:</strong> We applied option <code>nowarn</code> to
    <code>bootstrap</code> to suppress a warning message about the estimation
    sample not being marked. When using the <code>bootstrap</code>, one
    typically wants to restrict the resampling to observations that are
    actually used by the bootstrapped command (the "estimation sample"). This
    is why <code>bootstrap</code> checks whether the estimation sample has been
    marked (by looking at whether <code>e(sample)</code> is defined). We did
    not bother constructing the <code>e(sample)</code> in the example above
    because we know that all observations in the dataset will be used by
    <code>mysmithwelch</code> (remember that we dropped all observations with
    missing values at the beginning of the exercise). If there were
    observations in the dataset that are not used by <code>mysmithwelch</code>
    it would be important either to drop these observations before applying the
    bootstrap or to modify <code>mysmithwelch</code> such that it identifies
    the estimation sample.
</p>
<p>
    Overview table:
</p>
***/

esttab b1995m b1995f b2015m b2015f, mtitle nonum drop(dEC: :EC)

/***
<p>
    The overall change in the gender wage gap can be obtained from the bootstrap
    results as follows (we can use any of the variants as the point estimate
    for the overall change will be the same; the standard error will be slightly
    different depending on variant due to the random error of bootstrap
    estimation):
</p>
***/

estimates restore b1995m
lincom _b[dE:D] + _b[dC:D] + _b[dEC:D]

/***
<p>
    Interpretation: Hardly any of the results are statistically significant,
    which is not very surprising since already the small overall decrease in
    the gender wage gap over time is far from being significant (although a
    non-significant overall change does not imply that the decomposition
    components will not be significant; the different components may offset
    each other). If, in fact, there were systematic changes over time, these
    changes have likely been small and the sample at hand does not seem to have
    sufficient statistical power to detect them. Only for "dE:C" we find some
    weak evidence of a systematic effect.
</p>
***/

/***
<h2>Task 3: interventionist decomposition</h2>
<p><b>
    Compute the “interventionist” decomposition proposed by Kröger and
    Hartmann (2021) (see the slides) using 1995 as the starting
    point and interpret the results. Optionally provide bootstrap standard
    errors.
</b></p>
<p><i>
    Hint: You can try to use the <code>xtoaxaca</code> command provided by
    Kröger and Hartmann (type <code>findit xtoaxaca</code>). However, we find
    it more transparent to do this decomposition “by hand” using the formulas
    on the slides. A further alternative is to compute the “interventionist”
    decomposition from the results returned by <code>smithwelch</code>.
</i></p>
***/

/***
<p>
    We put together a small program that computes all required regressions and
    also collects the means, and then computes the results using matrix
    multiplication. This way it is easy to apply <code>bootstrap</code>.
</p>
***/

webdoc stlog, cmdlog
capt prog drop mydecomp
program mydecomp, eclass
    local y lnwage
    local x yeduc expft expft2
    local wgt [pw=weight]
    tempname bm0 bf0 bm1 bf1 Xm0 Xf0 Xm1 Xf1
    // males in 1995
    regress `y' `x' `wgt' if t==0 & female==0 
    matrix `bm0' = e(b)'
    mean `x' `wgt' if e(sample)
    matrix `Xm0' = e(b), 1
    // females in 1995
    regress `y' `x' `wgt' if t==0 & female==1
    matrix `bf0' = e(b)'
    mean `x' `wgt' if e(sample)
    matrix `Xf0' = e(b), 1
    // males in 2015
    regress `y' `x' `wgt' if t==1 & female==0 
    matrix `bm1' = e(b)'
    mean `x' `wgt' if e(sample)
    matrix `Xm1' = e(b), 1 
    // females in 2015
    regress `y' `x' `wgt' if t==1 & female==1 
    matrix `bf1' = e(b)'
    mean `x' `wgt' if e(sample)
    matrix `Xf1' = e(b), 1
    // decomposition
    tempname b
    matrix `b' = ///
        (`Xm1'-`Xm0')*`bm0' - (`Xf1'-`Xf0')*`bf0', ///
        `Xm0'*(`bm1'-`bm0') - `Xf0'*(`bf1'-`bf0'), ///
        (`Xm1'-`Xm0')*(`bm1'-`bm0') - (`Xf1'-`Xf0')*(`bf1'-`bf0')
    matrix `b' = `b', `b'[1,1] + `b'[1,2] + `b'[1,3]
    matrix colnames `b' = "Endowments" "Coefficients" "Interactions" "Total"
    ereturn post `b'
end
webdoc stlog close

/***
<p>
    We now apply <code>bootstrap</code> to the program (see above for 
    remarks on the <code>nowarn</code> option).
</p>
***/

bootstrap, cluster(psu) strata(strata) reps(100) nowarn: mydecomp

/***
<p>
    The results indicate that the gender wage gap decreases if the
    gender-specific levels of schooling and work experience found in 1995 are
    adjusted to those found in 2014. Conversely, if we adjusted the
    gender-specific wage structure found in 1995 to the one found in 2015, the
    gender wage gap slightly increases. However, none of these results is
    statistically significant.
</p>
<p>
    Overall, the results seem to be at odds with the results from
    the Smith-Welch decomposition. However, note that the “interventionist”
    decomposition has a somewhat different perspective than the Smith-Welch
    decomposition. The starting point of the Smith-Welch decomposition are two
    cross-sectional Oaxaca-Blinder decompositions and the goal is to explain
    how the differences in the explained part and in the unexplained part of
    these decompositions come about (i.e. the difference in the explained part
    and the difference in the unexplained part are separately decomposed into a
    part due to differences in endowments and a part due to differences in
    coefficients). The starting point of the “interventionist” decomposition are
    the overall gaps in two cross-sections and the goal is to show how much of the 
    difference in the gaps is due to a different configuration of endowments and 
    how much is due to a different configuration of coefficients.
</p>
<p>
    Despite the different perspectives, looking at the formulas we see that the
    (threefold) Smith-Welch decomposition computes all components
    required for the “interventionist” decomposition. The sums of the columns
    in the last table of the Smith-Welch output are equal to the components
    of the “interventionist” decomposition. So we can simply use 
    <code>smithwelch</code> to compute the “interventionist” decomposition:
</p>
***/

quietly svy: regress lnwage yeduc expft expft2 if female==0 & t==0
estimates store male_t0
quietly svy: regress lnwage yeduc expft expft2 if female==1 & t==0
estimates store female_t0
quietly svy: regress lnwage yeduc expft expft2 if female==0 & t==1
estimates store male_t1
quietly svy: regress lnwage yeduc expft expft2 if female==1 & t==1
estimates store female_t1
smithwelch male_t0 female_t0 male_t1 female_t1 
matrix DD = r(DD)
display "Endowments   = " DD[1,"dE:E"]  + DD[1,"dC:E"]  + DD[1,"dEC:E"]
display "Coefficients = " DD[1,"dE:C"]  + DD[1,"dC:C"]  + DD[1,"dEC:C"]
display "Interactions = " DD[1,"dE:EC"] + DD[1,"dC:EC"] + DD[1,"dEC:EC"]
display "Total        = " DD[1,"dE:D"]  + DD[1,"dC:D"]  + DD[1,"dEC:D"]

/***
<p>
    We could also specify option <code>reference(1)</code> or 
    <code>reference(2)</code> to reduce the number of terms to be 
    added up. Results will stay the same:
</p>
***/

smithwelch male_t0 female_t0 male_t1 female_t1, reference(1)
matrix DD = r(DD)
display "Endowments   = " DD[1,"dE:E"]  + DD[1,"dC:E"]
display "Coefficients = " DD[1,"dE:C"]  + DD[1,"dC:C"]
display "Interactions = " DD[1,"dE:EC"] + DD[1,"dC:EC"]
display "Total        = " DD[1,"dE:D"]  + DD[1,"dC:D"]
smithwelch male_t0 female_t0 male_t1 female_t1, reference(2)
matrix DD = r(DD)
display "Endowments   = " DD[1,"dE:E"]  + DD[1,"dC:E"]
display "Coefficients = " DD[1,"dE:C"]  + DD[1,"dC:C"]
display "Interactions = " DD[1,"dE:EC"] + DD[1,"dC:EC"]
display "Total        = " DD[1,"dE:D"]  + DD[1,"dC:D"]

/***
<p>
    <b>Using <code>xtoaxaca</code>:</b> Kröger and Hartmann (2021) provide
    command <code>xtoaxaca</code> that can be used to compute the
    “interventionist” decomposition (as well as other types of
    difference-in-differences decompositions). The procedure is such that first
    a model including all data is estimated, including all interactions with
    the two grouping variables (in our case this is equivalent to estimating
    four separate models). The model has to be stored and <code>xtoaxaca</code>
    is then applied to this stored model. The results from above can be
    reproduced using <code>xtoaxaca</code> as follows:
</p>
***/

quietly regress lnwage c.(yeduc expft expft2)##i.female##i.t [pw=weight]
estimates store m1
xtoaxaca yeduc expft expft2, groupvar(female) groupcat(0 1) ///
    timevar(t) times(0 1) timeref(0) change(interventionist) model(m1) ///
    weights(weight)

