clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-6-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 6: Reweighting</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages: <code>fre</code>, <code>dstat</code>, <code>kmatch</code>
</p>
***/

webdoc toc 1 1

/***
<p>
    Set the seed of the random number generator for sake of reproducibility:
</p>
***/

set seed 65443

/***
<h2>Task 1: manual reweighting decomposition</h2>
<p><b>
    Repeat the reweighting decomposition from the slides for the <i>mean</i>
    and the <i>variance</i> of log wages between the private sector and the
    public sector (section “Example analysis”), but use the “opposite”
    counterfactual. That is, compute a decomposition in which the distribution
    of <i>X</i> in the public sector is adjusted to the distribution observed
    in the private sector (on the slides, the distribution of <i>X</i> in the
    private sector was adjusted to the distribution observed in the public
    sector). In addition to schooling and experience, include occupational
    status as measured by the international socio-economic index
    (<code>isei</code>) as a predictor.
</b></p>
<p>
    Data preparation including <code>isei</code>:
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
summarize wage lnwage yeduc expft expft2 isei public
drop if missing(lnwage, yeduc, expft, isei, public) // remove unused observation
svyset psu [pw=weight], strata(strata)

/***
<p>
    Propensity-score model: We first estimate a model that includes quadratic
    terms for experience and ISEI as well as all two-way interactions between
    schooling, experience, and ISEI.
</p>
***/

svy: logit public ///
    /// main effects
    yeduc c.expft##c.expft c.isei##c.isei ///
    /// education x experience
    c.yeduc#c.expft c.yeduc#c.expft#c.expft /// 
    /// education x ISEI
    c.yeduc#c.isei c.yeduc#c.isei#c.isei ///
    /// ISEI x experience
    c.isei#c.expft c.isei#c.expft#c.expft ///
        c.isei#c.isei#c.expft ///
        c.isei#c.isei#c.expft#c.expft

/***
<p>
    The model appears hopelessly overspecified. Here is a joint test of the
    higher order terms of the interactions between ISEI and education or
    experience.
</p>
***/

test c.yeduc#c.isei#c.isei ///
    c.isei#c.expft#c.expft ///
    c.isei#c.isei#c.expft ///
    c.isei#c.isei#c.expft#c.expft

/***
<p>
    It seems we can safely drop these terms. We rerun the model without these
    terms.
</p>
***/

svy: logit public ///
    /// main effects
    yeduc c.expft##c.expft c.isei##c.isei ///
    /// education x experience
    c.yeduc#c.expft c.yeduc#c.expft#c.expft /// 
    /// education x ISEI and ISEI x experience
    c.yeduc#c.isei c.isei#c.expft

/***
<p>
    The model still appears overspecified. Let's further simplify by dropping
    the interactions between ISEI and education or experience entirely.
</p>
***/

test c.yeduc#c.isei c.isei#c.expft
svy: logit public c.yeduc##c.expft##c.expft c.isei##c.isei

/***
<p>
    Even the quadratic effect of ISEI seems to be unnecessary. So we will
    simply add a linear term for ISEI (note that also the model from the slides
    could be simplified, but we leave these terms in the model for sake of
    comparability). Our final model thus is as follows:
</p>
***/

svy: logit public c.yeduc##c.expft##c.expft isei

/***
<p>
    We now generate predicted probabilities from the model (i.e. probability of
    being in the public sector given the values of the covariates), compute the
    proportion of observations in the public sector, and compute the weights to
    reweight the public sector so that it looks like the private sector (the
    formula for the weights is flipped compared to the formula on the slides,
    where we reweighted the private sector).
</p>
***/

predict double PS, pr
summarize public [aw=weight] 
scalar Pr_public = r(mean)
generate double PSI = ((1-PS) / (1-Pr_public)) / (PS / Pr_public) if public==1
replace PSI = 1 if public==0
summarize PSI [aw=weight] if public==1

/***
<p>
    We can now compute the mean and variance of log wages in both groups as well
    as in the reweighted public sector and then compute the decomposition.
</p>
***/

// prepare a matrix for the results
matrix R = J(2,3,.)
matrix coln R = private public rw_public
matrix rown R = mean var
matrix list R
// raw results for private sector
summarize lnwage [aw=weight] if public==0
matrix R[1,1] = r(mean)
matrix R[2,1] = r(Var)
// raw results for public sector
summarize lnwage [aw=weight] if public==1
matrix R[1,2] = r(mean)
matrix R[2,2] = r(Var)
// reweighted results for public sector
summarize lnwage [aw=PSI*weight] if public==1
matrix R[1,3] = r(mean)
matrix R[2,3] = r(Var)
// obtain decomposition
mat list R
di as txt "difference in mean: total           = " as res R[1,1] - R[1,2] _n ///
   as txt "difference in mean: explained       = " as res R[1,3] - R[1,2] _n ///
   as txt "difference in mean: unexplained     = " as res R[1,1] - R[1,3] _n ///
   as txt "difference in variance: total       = " as res R[2,1] - R[2,2] _n ///
   as txt "difference in variance: explained   = " as res R[2,3] - R[2,2] _n ///
   as txt "difference in variance: unexplained = " as res R[2,1] - R[2,3]

/***
<p>
    Results are not directly comparable to the results on the slides because we
    added an extra predictor (ISEI; and also the number of observations changed
    slightly due to missings for ISEI). The raw difference now is slightly
    higher (0.139) and we see that less of the public-private gap in the mean
    of log wages is explained (on the slides it was 0.127; here it is 0.106).
    The remaining unexplained difference is 0.032. The interpretation is that
    the gap would only be 0.032 if people in the public sector had the same
    distribution of education, work experience and ISEI as the people in the
    private sector (i.e. people in the public sector would, on average, only
    earn about 3% more, not 14% more).
</p>
<p>
    For the variance we see that the explained part is slightly negative
    (similar as on slides).That is, wage inequality within the public sector
    would be even somewhat smaller than it actually is if people in the public
    sector had same distribution of education, work experience and ISEI as the
    people in the private sector.
</p>
***/

/***
<h2>Task 2: reweighting decomposition using dstat</h2>
<p><b>
    Try to replicate the decomposition using command <code>dstat</code>.
</b></p>
<p>
    We use option <code>rif()</code> to store the RIFs, and then compute the
    decomposition using these variables (see slides). An advantage of using RIFs
    is that consistent standard errors can easily be computed.
</p>
***/

// get RIFs of raw results
dstat (mean variance) lnwage, over(public) vce(svy) ///
    rif(RIF_0_mean RIF_0_var RIF_1_mean RIF_1_var)
// get RIFs of reweighted results (only the public sample is reweighted; the
// RIFs for private are the same as above)
dstat (mean variance) lnwage, over(public) vce(svy) ///
    balance(ipw: c.yeduc##c.expft##c.expft c.isei, reference(0)) ///
    rif(RIF_0_mean RIF_0_var RIF_c_mean RIF_c_var) replace
// compute the RIFs of the decomposition components
generate double mean_diff = RIF_0_mean - RIF_1_mean
generate double mean_expl = RIF_c_mean - RIF_1_mean
generate double mean_unex = RIF_0_mean - RIF_c_mean
generate double var_diff  = RIF_0_var  - RIF_1_var
generate double var_expl  = RIF_c_var  - RIF_1_var
generate double var_unex  = RIF_0_var  - RIF_c_var
// results
svy: mean mean_* var_* 
ereturn display // redisplay results including t-statistics and p values
drop RIF_* mean_* var_*

/***
<p>
    Note: The unexplained part of the decomposition could also be obtained
    directly by <code>dstat</code>.
</p>
***/

dstat (mean variance) lnwage, over(public, contrast(1)) vce(svy) ///
    balance(ipw: c.yeduc##c.expft##c.expft c.isei, reference(0))


/***
<h2>Task 3: bootstrap standard errors</h2>
<p><b>
    Wrap the analysis from task 1 into a small program and apply the
    bootstrap to compute standard errors and confidence intervals. How do the
    bootstrap standard errors compare to the analytic standard errors provided
    by <code>dstat</code>?
</b></p>
***/

/***
<p>
    We write a program that allows us to specify the predictors to be included
    in the reweighting and also allows us to specify the summary statistics to
    be decomposed (any statistic returned by <code>summarize</code> can be
    used; see "Stored results" in <code>help summarize</code>). The program
    also marks the estimation sample so that observations with missing value on
    any of the relevant variables will be excluded from all computations (and
    so that <code>bootstrap</code> will know which observations have been used).
</p>
***/

webdoc init, cmdlog
capture program drop myreweight
program myreweight, eclass
    syntax varlist(fv) [, Stats(str) ]
    if "`stats'"=="" local stats "mean" // default
    // generate weights
    logit public `varlist' [pw=weight]
    local N = e(N) // number of observations
    tempvar touse 
    generate byte `touse' = e(sample) // this marks the estimation sample
    tempvar PS PSI
    qui predict double `PS' if `touse', pr
    summarize public [aw=weight] if `touse', meanonly
    quietly generate double `PSI' = ///
        ((1-`PS') / (1-r(mean))) / (`PS' / r(mean)) ///
        if public==1 & `touse'
    // compute results
    // - setup results vector
    local nstats: list sizeof stats
    tempname b
    matrix `b' = J(1, 6*`nstats', .)
    local colnames
    foreach stat of local stats {
        local colnames `colnames' ///
            `stat':private `stat':public `stat':rw_public ///
            `stat':difference `stat':explained `stat':unexplained
    }
    matrix coln `b' = `colnames'
    // raw results for private sector
    quietly summarize lnwage [aw=weight] if public==0 & `touse', detail
    local j = 1
    foreach stat of local stats {
        matrix `b'[1,`j'] = r(`stat')
        local j = `j' + 6
    }
    // raw results for public sector
    quietly summarize lnwage [aw=weight] if public==1 & `touse', detail
    local j = 2
    foreach stat of local stats {
        matrix `b'[1,`j'] = r(`stat')
        local j = `j' + 6
    }
    // reweighted results for public sector
    quietly summarize lnwage if public==1 & `touse' [aw=`PSI'*weight], detail
    local j = 3
    foreach stat of local stats {
        matrix `b'[1,`j'] = r(`stat')
        local j = `j' + 6
    }
    // decomposition
    local j = 0
    foreach stat of local stats {
        matrix `b'[1,`j'+4] = `b'[1,`j'+1] - `b'[1,`j'+2]
        matrix `b'[1,`j'+5] = `b'[1,`j'+3] - `b'[1,`j'+2]
        matrix `b'[1,`j'+6] = `b'[1,`j'+1] - `b'[1,`j'+3]
        local j = `j' + 6
    }
    // post results
    ereturn post `b', obs(`N') esample(`touse')
    ereturn display
end
webdoc init, nocmdlog

/***
<p>
    We can now use the program. Here is an example of a single run where we
    decompose the difference in the mean, the 25% quantile, the median, the 75%
    quantile, and the variance:
</p>
***/

myreweight c.yeduc##c.expft##c.expft c.isei, stats(mean p25 p50 p75 Var)

/***
<p>
    Results for the mean and the variance are identical to above. Looking at
    the results for quantiles we see that the public-private wage gap is larger
    at the bottom of the distribution, mostly due to the unexplained part.
    However, relevance of the unexplained part is changing from p25 to p50 and
    p75. For the 75% quantile the unexplained part even switches sign.
</p>
<p>
    Now use the bootstrap to obtain standard errors and confidence intervals
    for the results. We only include the mean and the variance.
</p>
***/

bootstrap, cluster(psu) reps(100): ///
    myreweight c.yeduc##c.expft##c.expft c.isei, stats(mean Var)

/***
<p>
    Results are very similar to the ones obtained using <code>dstat</code>. For the
    mean, the explained part is highly significant; the unexplained part is not
    significantly different from zero. For the variance, results are opposite.
</p>
***/

/***
<h2>Task 4: reweighting decomposition using entropy balancing</h2>
<p><b>
    Evaluate how results change if you use entropy balancing to compute the
    weights instead of IPW based on a <code>logit</code> model.
</b></p>
***/

/***
<p>
    We can use the same code as in task 2 and simply replace <code>ipw</code> by
    <code>eb</code>.
</p>
***/

// get RIFs of raw results
dstat (mean variance) lnwage, over(public) vce(svy) ///
    rif(RIF_0_mean RIF_0_var RIF_1_mean RIF_1_var)
// get RIFs of reweighted results (only the public sample is reweighted; the
// RIFs for private are the same as above)
dstat (mean variance) lnwage, over(public) vce(svy) ///
    balance(eb: c.yeduc##c.expft##c.expft c.isei, reference(0)) ///
    rif(RIF_0_mean RIF_0_var RIF_c_mean RIF_c_var) replace
// compute the RIFs of the decomposition components
generate double mean_diff = RIF_0_mean - RIF_1_mean
generate double mean_expl = RIF_c_mean - RIF_1_mean
generate double mean_unex = RIF_0_mean - RIF_c_mean
generate double var_diff  = RIF_0_var  - RIF_1_var
generate double var_expl  = RIF_c_var  - RIF_1_var
generate double var_unex  = RIF_0_var  - RIF_c_var
// results
svy: mean mean_* var_* 
ereturn display // redisplay results including t-statistics and p values
drop RIF_* mean_* var_*

/***
<p>
    Decomposition results did not change all that much. For both the mean and
    the variance, the absolute value of the explained part slightly increased.
    For the variance, the explained part is now marginally significant.
</p>
***/

/***
<h2>Task 5: reweighted Oaxaca-Blinder decomposition</h2>
<p><b>
    Compute a reweighted Oaxaca-Blinder decomposition to explain the
    public–private wage gap and interpret the results. Include schooling,
    experience, and international socio-economic index (ISEI) as predictors.
    Do the analysis in both directions (i.e. once reweighting the private
    sector and once reweighting the public sector).
</b></p>
***/

/***
<p>
    Compute the weights, coefficients, and means:
</p>
***/

// compute weights
kmatch ipw public c.yeduc##c.expft##c.expft c.isei [pw=weight], att wgen(PSI0)
kmatch ipw public c.yeduc##c.expft##c.expft c.isei [pw=weight], atc wgen(PSI1)
// obtain coefficients and means in private sector
svy: regress lnwage yeduc expft expft2 isei if public==0
mat b_priv = e(b)
svy: mean yeduc expft expft2 isei if public==0
mat X_priv = (e(b),1)
// obtain counterfactual coefficients and means in private sector
regress lnwage yeduc expft expft2 isei [pw=PSI0] if public==0
mat b_priv_C = e(b)
mean yeduc expft expft2 isei [pw=PSI0] if public==0
mat X_priv_C = (e(b),1)
// obtain coefficients and means in public sector
svy: regress lnwage yeduc expft expft2 isei if public==1
mat b_pub = e(b)
svy: mean yeduc expft expft2 isei if public==1
mat X_pub = (e(b),1)
// obtain counterfactual coefficients and means in public sector
regress lnwage yeduc expft expft2 isei [pw=PSI1] if public==1
mat b_pub_C = e(b)
mean yeduc expft expft2 isei [pw=PSI1] if public==1
mat X_pub_C = (e(b),1)

/***
<p>
    Variable <code>PSI0</code> reweights the private sector so that it looks
    like the public sector. Variable <code>PSI1</code> reweights the public
    sector so that it looks like the private sector.
</p>
***/

/***
<p>
    Here are the results for the decomposition that reweights the private sector:
</p>
***/

mat D_priv_C = (X_priv * b_priv' - X_pub * b_pub') ///
             \ (X_priv - X_priv_C) * b_priv'       ///
             \  X_priv_C * (b_priv - b_priv_C)'    ///
             \  X_pub * (b_priv_C - b_pub)'        ///
             \ (X_priv_C - X_pub) * b_priv_C'
mat rown D_priv_C = "Overall difference" "Explained" "Specification error" ///
              "Unexplained" "Reweighting error"
matlist D_priv_C, twidth(20)

/***
<p>
    Here are the results for the decomposition that reweights the public sector:
</p>
***/

mat D_pub_C = (X_priv * b_priv' - X_pub * b_pub') ///
            \ (X_pub_C - X_pub) * b_pub'          ///
            \  X_pub_C * (b_pub_C - b_pub)'       ///
            \  X_priv * (b_priv - b_pub_C)'       ///
            \ (X_priv - X_pub_C) * b_pub_C'
mat rown D_pub_C = "Overall difference" "Explained" "Specification error" ///
             "Unexplained" "Reweighting error"
matlist D_pub_C, twidth(20)

/***
<p>
    Note that command <code>oaxaca_rif</code> (from package <code>rif</code> by
    Fernando Rios-Avila; type <code>ssc install rif</code>) can be used to
    compute reweighted OB decompositions automatically (but be aware that the standard errors
    reported by <code>oaxaca_rif</code> do not take account of the fact that
    the weights in the reweighting step are estimated; so you might want to use
    the bootstrap for the standard errors).
</p>
***/



