clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-7-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 7: RIF decomposition</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages: <code>rifreg</code> (from 
    <a href="https://sites.google.com/view/nicole-m-fortin/data-and-programs">N. Fortin's website</a>),
    <code>rif</code>, <code>oaxaca</code>, <code>dstat</code>, <code>kmatch</code>, <code>moremata</code>
</p>
***/

webdoc toc 1 1

/***
<h2>Task 1: RIF decomposition</h2>
<p><b>
    Repeat the example analysis of the private–public gap in wage inequality.
    This time, use the Gini coefficient as well as the D9/D1 ratio, the D5/D1
    ratio, and the D9/D5 ratio as inequality measures. If possible, use
    <code>rifreg</code>, <code>rifvar</code>, and <code>oaxaca_rif</code> to
    calculate these results.
</b></p>
***/

/***
<p>
    Data preparation as on slides:
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
svyset psu [pw=weight], strata(strata)
summarize wage lnwage yeduc expft expft2 public
drop if missing(lnwage, yeduc, expft, public)

/***
<p>
    We first do the decomposition of the Gini coefficient (using
    variable <code>wage</code>, not <code>lnwage</code>).
</p>
<ul>
<li>
    using <code>rifreg</code> and <code>oaxaca</code>
</li>
***/

rifreg wage [aw=weight] if public==0, gini retain(RIFprivate)
rifreg wage [aw=weight] if public==1, gini retain(RIFpublic)

/*** ***/

generate double RIF = cond(public==1, RIFpublic, RIFprivate)
oaxaca RIF yeduc (experience: expft expft2), by(public) weight(1) svy
drop RIF*

/***
<li>
    using <code>rifvar</code> and <code>oaxaca</code>
</li>
***/

egen double RIF = rifvar(wage), gini by(public) weight(weight)
oaxaca RIF yeduc (experience: expft expft2), by(public) weight(1) svy 
drop RIF

/***
<li>
    using <code>oaxaca_rif</code> (this is the same as applying
    <code>rifvar</code> followed <code>oaxaca</code>; however, option
    <code>svy</code> is not supported by <code>oaxaca_rif</code>, so we only
    take account of weights and PSUs)
</li>
***/

oaxaca_rif wage yeduc (experience: expft expft2) [pw=weight], by(public) ///
    wgt(1) rif(gini) cluster(psu)

/***
</ul>
<p>
    Interpretation: There is a difference of 0.057 in the Gini coefficient
    (higher wage inequality in the private sector). This difference cannot be
    explained by compositional differences w.r.t. years of education and
    full-time employment experience. There is some weak evidence that the gap
    would even be larger if composition was the same in the two sectors. Thus,
    wage inequality is higher in the private sector because wage-setting
    mechanisms in the private sector are more inequality enhancing than they
    are in the public sector.
</p>
***/

/***
<p>
    We now look at D9/D1, D5/D1, and D9/D5. These measures are not supported by
    <code>rifreg</code>, so we only use the approach based on
    <code>rifvar</code> followed by <code>oaxaca</code>. Furthermore, we
    analyze the inter-quantile range of <code>lnwage</code> (rather than the
    inter-quantile ratio of <code>wage</code>).
</p>
<ul>
<li>D9/D1</li>
***/

egen double RIF = rifvar(lnwage), iqr(10 90) by(public) weight(weight)
oaxaca RIF yeduc (experience: expft expft2), by(public) weight(1) svy 
drop RIF

/***
<p>
    Interpretation: D9/D1 is about 0.19 log points larger in the private sector
    (i.e., higher wage inequality in the private sector). This difference
    cannot be explained by compositional differences w.r.t. years of education
    and full-time employment experience. Quite the opposite: If workers in the
    public sector were paid like workers in the private sector, we would expect
    a higher wage inequality in the public sector than in the private sector.
</p>
***/

/***
<li>D5/D1</li>
***/

egen double RIF = rifvar(lnwage), iqr(10 50) by(public) weight(weight)
oaxaca RIF yeduc (experience: expft expft2), by(public) weight(1) svy 
drop RIF

/***
<p>
    Interpretation: There is almost no difference in D5/D1 of the private sector
    and the public sector (i.e., wage inequality in the lower half of the
    wage distributions in the private sector and in the public sector is very
    similar). However, we see that if workers in the public sector were paid
    like workers in the private sector, we would expect a higher wage
    inequality in the lower half of the wage distribution in the public than in
    the private sector.
</p>
***/

/***
<li>D9/D5</li>
***/

egen double RIF = rifvar(lnwage), iqr(50 90) by(public) weight(weight)
oaxaca RIF yeduc (experience: expft expft2), by(public) weight(1) svy 
drop RIF

/***
<p>
    Interpretation: D9/D5 is about 0.19 log points larger in the private sector
    (i.e., higher wage inequality in the private sector than in the public
    sector if we look at the upper half of the wage distribution). This
    difference cannot be explained by compositional differences w.r.t. years of
    education and full-time employment experience. There is a small and
    marginally significant effect of education, suggesting that if workers in
    the public sector were paid like workers in the private sector, we would
    expect a (slightly) higher wage inequality in the upper half of the wage
    distribution in the public sector than in the private sector.
</p>
</ul>
<p>
    Overall, the pattern is such that the difference in wage inequality between
    the public and the private sector cannot be explained by compositional
    differences with respect to education and work experience. If anything, the
    gap would even be larger if the two sectors had the same composition (this
    result is mostly related to education; if in the private sector average
    level of education would be as high as in the public sector, wage inequality
    in the private sector would even be higher). We also see that the gap in
    wage inequality is mostly driven by the upper half of the distribution;
    yet, also in the lower part of the distribution compositional differences
    seem to have a suppressing effect on the inequality gap.
</p>
***/

/***
<h2>Task 2: reweighted RIF decomposition</h2>
<p><b>
    Combine your RIF decomposition for the Gini coefficient with reweighting
    (analogous to the reweighted OB decomposition) and calculate the
    specification error. Use <code>oaxaca_rif</code> for this exercise.
</b></p>
***/

oaxaca_rif wage yeduc (experience: expft expft2) ///
    [pw=weight], by(public) cluster(psu) wgt(1) rif(gini) ///
    rwlogit(c.yeduc##c.expft##c.expft)

/***
<p>
    Interpretation: In total, the specification error is not significant, but
    there is some weak evidence for a misspecified effect regarding education.
    The reweighting error is very small.
</p>
***/


/***
<h2>Task 3: manual reweighted RIF decomposition</h2>
<p><b>
    Try to replicate the results for the reweighted Gini decomposition manually
    by first computing the RIF and then applying <code>oaxaca</code> to the
    RIF taking reweighting into account. You will need two calls to
    <code>oaxaca</code> to compute all results.
</b></p>
***/

/***
<p>
    Step 1: Generate the sector-specific RIF of the Gini using command
    <code>dstat</code> with option <code>rif()</code>.
</p>
***/

dstat (gini) wage [pw=weight], over(public) rif(RIF, compact)
svy: mean RIF, over(public)

/***
<p>
    Step 2: Generate balancing weights that adjust the distribution of
    covariates among people in the private sector to the distribution observed
    among people in the public sector (i.e. the private sector is reweighted)
    using <code>kmatch</code>.
</p>
***/

kmatch ipw public c.yeduc##c.expft##c.expft [pw=weight], att wgen(ipw)
bysort public: summarize ipw 

/***
<p>
    Step 3: Compute the reweighted RIF of the Gini for the private sector
    using <code>dstat</code>.
</p>
***/

dstat (gini) wage if public==0 [pw=ipw], rif(RIFipw)

/***
<p>
    Step 4: Apply <code>oaxaca</code> to the raw RIF of private sector and the
    reweighted RIF of the private sector to obtain the “pure explained part”
    and the “specification error” (the “explained part” of this decomposition
    quantifies the “pure explained part”; the “unexplained part” quantifies the
    “specification error”).
</p>   
***/

// preserve the data so that they can be restored later
preserve
// add a unique ID for each observation
generate ID = _n
// duplicate each private sector observation
expand 2 if public==0
// generate a 0/1 variable that tags the duplicates; we can then use this as
// the group variable in oaxaca
bysort ID: generate byte G = (_n==2) if public==0
// generate weights containing 1 (or weights if survey weigths are applied) 
// for G==0 and the balancing weights for G==1
replace ipw = weight if G==0 
// generate a RIF variable containing the raw RIF for G==0 and the reweighted
// RIF for G==1
replace RIF = RIFipw if G==1
// apply oaxaca to the RIF variable while applying the weights; G is the group
// variable
oaxaca RIF yeduc (experience: expft expft2) [pw=ipw], by(G) weight(1) cluster(ID)
estimates store IPW1
// restore original data
restore

/***
<p>
    Step 5: Apply <code>oaxaca</code> to the reweighted RIF of the private
    sector and the raw RIF of public sector to obtain the “pure unexplained
    part” and the “reweighting error” (the “explained part” of this
    decomposition quantifies the “reweighting error”; the “unexplained part”
    quantifies the “pure unexplained part”).
</p>
***/

replace RIFipw = RIF if public==1 // fill in RIFipw for public sector
oaxaca RIFipw yeduc (experience: expft expft2) [pw=ipw], by(public) weight(1)
estimates store IPW2

/***
<p>
    Overview of results:
</p>
<ul>
    <li>pure explained part: results under “explained” in column 1</li>
    <li>specification error: results under "unexplained" in column 1</li>
    <li>pure unexplained part: results under “unexplained” in column 2</li>
    <li>reweighting error: results under "explained" in column 2</li>
</ul>
***/

esttab IPW1 IPW2, nogap mti

/***
<p>
    The point estimates are the same as the ones computed by
    <code>oaxaca_rif</code> above (standard errors are not reliable in either
    case; a quick comparison against the bootstrap indicates that some of the
    standard errors reported by <code>oaxaca_rif</code> are completely off; the
    standard errors obtained by the manual procedure seem to be more accurate,
    but still biased).
</p>
***/

/***
<h2>Task 4: using entropy balancing</h2>
<p><b>
    Repeat the analysis using entropy balancing for the reweighting. How do the
    results change?
</b></p>
***/

/***
<p>
    Step 1 can remain the same. In Step 2 we need to use entropy balancing to
    generate the weights. Then use these weights in the subsequent steps.
</p>
***/

// Step 2
kmatch eb public c.yeduc##c.expft##c.expft [pw=weight], att wgen(eb)
bysort public: summarize eb
// Step 3
dstat (gini) wage if public==0 [pw=eb], rif(RIFeb)
// Step 4
// preserve the data so that they can be restored later
preserve
// add a unique ID for each observation
generate ID = _n
// duplicate each private sector observation
expand 2 if public==0
// generate a 0/1 variable that tags the duplicates; we can then use this as
// the group variable in oaxaca
bysort ID: generate byte G = (_n==2) if public==0
// generate weights containing 1 (or weights if survey weigths are applied) 
//for G==0 and the balancing weights for G==1
replace eb = weight if G==0 
// generate a RIF variable containing the raw RIF for G==0 and the reweighted
// RIF for G==1
replace RIF = RIFeb if G==1
// apply oaxaca to the RIF variable while applying the weights; G is the group
// variable
oaxaca RIF yeduc (experience: expft expft2) [pw=eb], by(G) weight(1) cluster(ID)
estimates store EB1
// restore original data
restore
// Step 5
replace RIFeb = RIF if public==1
oaxaca RIFeb yeduc (experience: expft expft2) [pw=eb], by(public) weight(1)
estimates store EB2

/***
<p>
    Overview of results:
</p>
***/

esttab IPW1 IPW2 EB1 EB2, nogap mti

/***
<p>
    We see that the reweighting error is now zero (apart from roundoff error).
</p>
***/

