clear all
version 16.1
set linesize 100

/*
    To generate a HTML file including text, commands, and output, first install
    -webdoc- by typing
    
        . ssc install webdoc, replace
    
    After that, type
    
        . webdoc do Exercise-8-solutions.do, replace
    
    to compile the document.
    
    Pieces of text included in the document have to be annotated by HTML tags
    (see https://wiki.selfhtml.org/wiki/HTML#Elemente for an overview). Here are
    some examples:
    
        <h1>heading 1</h1>
        <h2>heading 2</h2>
        <h3>heading 3</h3>
        <p>a paragraph of text</p>
        <b>bold text</b>
        <i>italic text</i>
        <code>computer code</code>
*/

webdoc init, header(width(800px) stscheme(lcom)) logall nokeep gropts(hardcode as(svg) nokeep)
webdoc put <style>pre {max-height:500px}</style> // this limits the vertical size of output sections

/***
<h1>Decomposition Methods in the Social Sciences<h1>
<h1>Solutions to Exercise 8: Distribution decompositions</h1>

<p><b>
    Johannes Giesecke and Ben Jann, GESIS Training Course, January 29 – February 1, 2024
</b></p>
<p>
    Required packages: <code>cdist</code>, <code>dstat</code>, <code>estout</code>,
    <code>jmpierce</code>, <code>moremata</code>, <code>grstyle</code>, <code>palettes</code>, <code>colrspace</code>
</p>
***/

webdoc toc 1 1

/***
<p>
    Set the seed of the random number generator for sake of reproducibility:
</p>
***/

set seed 439028

/***
<h2>Task 1: extension of example analysis from slides</h2>
<p><b>
    Extend the model from the session on distribution decompositions.
    Include the international socio-economic index (<code>isei</code>) as well as
    the number of children in the household (<code>children</code>). Decompose the
    private–public gap in the D9/D1, the D9/D5 and the D5/D1 ratio. Use the
    approaches based on JMP, conditional quantiles and distribution regressions
    and compare the results. The decompositions should be such that the
    covariate distribution of the private sector is adjusted to the
    covariate distribution of the public sector (i.e. use the wage structure
    from the private sector as the reference wage structure).
</b></p>
***/

/***
<p>
    Data preparation including additional predictors:
</p>
***/

use gsoep-extract, clear
keep if wave==2015
keep if inrange(age, 25, 55)
generate lnwage = ln(wage)
generate expft2 = expft^2
summarize lnwage yeduc expft expft2 public isei children
drop if missing(lnwage, yeduc, expft, public, isei, children) // remove unused observation

/***
<p>
    Overview of characteristics:
</p>
***/

tabstat yeduc expft isei children [aw=weight], by(public)

/***
<p>
    People in the public sector are on average higher educated, have less full-
    time labor experience, higher occupational status and about the same number
    of children than people in the private sector.
</p>
<p>
    JMP decomposition:
</p>
***/

regress lnwage yeduc expft expft2 isei children [pw=weight] if public==0
estimates store private
regress lnwage yeduc expft expft2 isei children [pw=weight] if public==1
estimates store public
jmpierce private public, reference(1) statistics(mean d9010 d9050 d5010 variance)

/***
<p>
    We see that differences in average log wages between the private and the
    public sector can be fully explained by compositional differences between
    workers in both sectors (Q-component). However, if the private sector had the public
    sector's covariate distribution, overall wage inequality (as measured by
    D9D1) in the private sector would increase (i.e. the inequality-gap would
    be larger). Interestingly, at the same time, the sector-gap in the
    dispersion of wages in the upper half of the wage distribution (D9D5) is
    predicted to be a little lower than it actually is, whereas for the lower
    half of the wage distribution (D5D1) it is predicted to be much larger.
    Finally, with respect to the sector-gap in the variance of log wages,
    compositional differences cannot explain the observed gap.
</p>
<p>
    Looking at the P- and the U-components, we see that (with the exception of
    the mean) large parts of the inequality-gaps are due to a less
    "inequality-prone" wage structure in the public sector (component P) and
    less "inequality-prone" unobservable characteristics and wage returns to
    these characteristics.
</p>
<p>
    Decomposition based on conditional quantiles:
</p>
***/

// Estimate counterfactuals
cdist lnwage yeduc c.expft##c.expft2 isei children [pw=weight], by(public) method(qr) ///
    statistics(mean iqr(10 90) iqr(50 90) iqr(10 50) variance) ///
    vce(bootstrap, cluster(psu))

/*** ***/

// Decomposition with private sector wage structure as reference
cdist decomp
estimates store qr_priv

/***
<p>
    Decomposition based on distribution regression:
</p>
***/

// Estimate counterfactuals
cdist lnwage yeduc c.expft##c.expft2 isei children [pw=weight], by(public) ///
    statistics(mean iqr(10 90) iqr(50 90) iqr(10 50) variance) ///
    vce(bootstrap, cluster(psu))

/*** ***/

// Decomposition with private sector wage structure as reference
cdist decomp
estimates store dr_priv

/***
<p>
    We again see that the sector-gap in mean log wages can be fully explained
    by the compositional differences between workers in the private and in the
    public sector. In contrast, sector differences in wage inequality cannot be
    accounted for by compositional differences. The results again suggest that
    wage inequality in the private sector would even be larger than it actually
    is if this sector's covariate distribution was the same as the covariate
    distribution in the public sector.
</p>
***/

/***
<h2>Task 2: change in perspective</h2>
<p><b>
    How do results change if you adjust the covariates of people in the
    public to those of people in the private sector (i.e. if you use the wage
    structure from the public sector as the reference wage structure)?
</b></p>
***/


/***
<p>
    Decomposition based on conditional quantiles:
</p>
***/

// Decomposition with private sector wage structure as reference
estimates restore qr_priv
cdist decomp, reverse
estimates store qr_publ

/*** ***/

// Comparison
esttab qr_priv qr_publ, b(3) not nonum mti

/***
<p>
    Results in the explained part are a little less pronounced if we use the
    public sector wage structure as reference (i.e. adjusting the covariate
    distribution in the public sector to that of the private sector). However,
    the overall interpretation stays the same: The gap in mean log wages can at
    least to a large extent be explained by compositional differences, whereas
    the inequality-gaps cannot.
</p>
***/

/***
<p>
    Decomposition based on distribution regression:
</p>
***/

//Decomposition with public sector wage structure as reference
estimates restore dr_priv
cdist decomp, reverse
estimates store dr_publ

/*** ***/

// Comparison
esttab dr_priv dr_publ, b(3) not nonum mti

/***
<p>
    Similar as above.
</p>
***/


/***
<h2>Task 3: comparison to reweighting</h2>
<p><b>
    Optional: Compare your results to results from analogous decompositions
    using reweighting (e.g. compare the results from the distribution
    regression decomposition to results from decompositions based on IPW or
    entropy balancing). Can you reduce the difference between results by
    fine-tuning the models used in the various decompositions?
</b></p>
***/

/***
<p>
    To make things more convenient, let's write a small program to run
    reweighting decompositions:
***/

webdoc stlog, cmdlog
capt prog drop ipwdecomp
program ipwdecomp, eclass
    // syntax
    syntax varlist(fv min=2) [if] [in] [fw pw iw], by(varname) [ s(str) eb ]
    gettoken depvar controls : varlist
    if `"`eb'"'=="" local method ipw
    else            local method eb
    // estimation sample
    marksample touse
    markout `touse' `by'
    // counterfactual
    tempname adj0
    qui dstat (`s') `depvar' if `touse' [`weight'`exp'], over(`by') nose ///
        balance(`method':`controls', reference(1))
    local grps `"`e(over_namelist)'"'
    if `:list sizeof grps'!=2 {
        di as err "by() must dichotomous (two groups)"
        exit 498
    }
    local g0: word 1 of `grps'
    local g1: word 2 of `grps'
    matrix `adj0' = e(b)[1,`"`g0':"']
    // observed 
    tempname obs0 obs1
    qui dstat (`s') `depvar' if `touse' [`weight'`exp'], over(`by') nose
    matrix `obs0' = e(b)[1,`"`g0':"']
    matrix `obs1' = e(b)[1,`"`g1':"']
    // decomposition
    tempname b tmp
    matrix `b' = `obs0'-`obs1'
    matrix coleq `b' = "Difference"
    matrix `tmp' = `obs0' - `adj0'
    matrix coleq `tmp' = "Explained"
    matrix `b' = `b', `tmp'
    matrix `tmp' = `adj0' - `obs1'
    matrix coleq `tmp' = "Unexplained"
    matrix `b' = `b', `tmp'
    // post results
    eret post `b' [`weight'`exp'], depname(`depvar') esample(`touse') obs(`e(N)')
    eret local cmd "ipwdecomp"
    eret local eb "`eb'"
    // display
    eret display, vsquish
end
webdoc stlog close

/***
<p>
    The syntax is
</p>
<pre>
ipwdecomp <i>depvar</i> <i>indepvars</i> [<i>if</i>] [<i>in</i>] [<i>weight</i>], by(<i>groupvar</i>) [ s(<i>statistics</i>) eb ]
</pre>
<p>
    where <code><i>statistics</i></code> is a list of target statistics (any
    statistic supported by <code>dstat</code> is allowed) and option
    <code>eb</code> requests using entropy balancing rather than logit-based
    IPW.
</p>
<p>
    We now estimate several variants of the decomposition using the same
    specification as above for the covariates (linear terms for education, ISEI,
    and number of children, quadratic term for work experience, no
    interactions) and plot the results in a graph.
</p>
***/

local lhs yeduc c.expft##c.expft isei children
local stats mean iqr(10,90) iqr(50,90) iqr(10,50) variance
ipwdecomp lnwage `lhs' [pw=weight], by(public) s(`stats')
estimates store ipw
ipwdecomp lnwage `lhs' [pw=weight], by(public) s(`stats') eb
estimates store eb
cdist lnwage `lhs' [pw=weight], by(public) s(`stats') decomp
estimates store dr
cdist lnwage `lhs' [pw=weight], by(public) s(`stats') decomp method(qr)
estimates store qr

/*** ***/

grstyle init
grstyle set plain, grid
grstyle set color sb
grstyle set legend 3, inside nobox
coefplot ipw eb dr qr, noci keep(*:*) ///
    eqrename(Delta = Difference Chars = Explained Coefs = Unexplained) ///
    recast(bar) barwidth(.15) xline(0) ///
    plotlabels("logit IPW" "entropy balancing IPW" ///
               "distribution regression" "quantile regression")
webdoc graph

/***
<p>
    In the top panel the raw differences are reported. In principle, these should
    be the same for all methods. However, in the distribution and quantile
    regression approaches, fitted raw differences are computed which are
    affected by approximation error. For the mean, the approximation seems to
    be good, but for some of the inequality measures there are substantial
    deviations, particularly when using the quantile regression approach.
</p>
<p>
    With respect to the breakup into an explained and an unexplained component,
    logit IPW and entropy balancing pretty much agree. Results from distribution
    regression and quantile regression deviate here an there, but the overall picture 
    is similar.
</p>
<p>
    We now check whether differences between the results can be reduced by
    using a more flexible specification for the covariates. Here are the
    results we obtain if we include all two-way interaction and a squared terms
    for each variable.
</p>
***/

local lhs c.yeduc##c.expft##c.isei##c.children/*
    */ c.yeduc#c.yeduc c.expft#c.expft/*
    */ c.isei#c.isei c.children#c.children 
local stats mean iqr(10,90) iqr(50,90) iqr(10,50) variance
ipwdecomp lnwage `lhs' [pw=weight], by(public) s(`stats')
estimates store ipw
ipwdecomp lnwage `lhs' [pw=weight], by(public) s(`stats') eb
estimates store eb
cdist lnwage `lhs' [pw=weight], by(public) s(`stats') ///
    lincom((Difference:fit0-fit1) (Explained:fit0-adj0) (Unexplained:adj0-fit1))
estimates store dr
cdist lnwage `lhs' [pw=weight], by(public) s(`stats') method(qr) ///
    lincom((Difference:fit0-fit1) (Explained:fit0-adj0) (Unexplained:adj0-fit1))
estimates store qr
coefplot ipw eb dr qr, noci keep(*:*) ///
    eqrename(Delta = Difference Chars = Explained Coefs = Unexplained) ///
    recast(bar) barwidth(.15) xline(0) ///
    plotlabels("logit IPW" "entropy balancing IPW" ///
               "distribution regression" "quantile regression")
webdoc graph

/***
<p>
    The agreement of results across methods became somewhat better, but the 
    changes are not dramatic. Some approximation error still remains for the
    distribution regression and quantile regression approaches, and there are
    still some differences in the explained–unexplained breakup across the methods.
</p>
<p>
    Which of the results are more appropriate is hard to say. We are not aware
    of any research systematically comparing the approaches (e.g., using
    simulations), to find out whether some approaches generally (or under some
    conditions) outperform others. One might expect that the approaches based
    on distribution regression and quantile regression perform better than
    reweighting because they flexibly model the counterfactual distribution.
    However, the fact that the results from these two approaches do not agree very
    well makes us skeptical whether this is indeed the case.
</p>
<p>
    Yet, despite the discussed differences, the results from the various
    methods are qualitatively similar. Essentially all of the private-public
    gap in average wages is accounted for by differential distributions of
    characteristics, but the gap in wage inequality within the sectors remains
    largely unexplained. If anything, wage inequality in private sector would
    even be larger if the private sector had a distribution of characteristics
    like the public sector, and we see that mostly the bottom half of the
    distribution would be affected.
</p>
***/




